%Process Capacitor charge & discharge data
% LAST UPDATED by Andy French Mar 2020
function process_data

%Fontsize for graphs
fsize = 16;

%Load imported data
load('capacitor data')

%Initialize charge, discharge and line of best fit curves
fig_Vcharge = figure; ax_V_charge = axes('nextplot','add'); grid on; set(gca,'fontsize',fsize);
xlabel('t/s','fontsize',fsize); ylabel('Capacitor voltage /volts','fontsize',fsize); 
title('Capacitor charging voltage vs time','fontsize',fsize)
fig_Vdischarge = figure; ax_V_discharge = axes('nextplot','add'); grid on; set(gca,'fontsize',fsize);
xlabel('t/s','fontsize',fsize); ylabel('Capacitor voltage /volts','fontsize',fsize); 
title('Capacitor discharging voltage vs time','fontsize',fsize)
fig_Idischarge = figure; ax_I_discharge = axes('nextplot','add'); grid on; set(gca,'fontsize',fsize);
xlabel('t/s','fontsize',fsize); ylabel('Current /mA','fontsize',fsize); 
title('Capacitor discharging current vs time','fontsize',fsize)
fig_best_fit = figure; ax_best_fit = axes('nextplot','add'); grid on; set(gca,'fontsize',fsize);
xlabel('t/s','fontsize',fsize); ylabel('ln(V/volts)','fontsize',fsize); 
title('ln(V) vs t line of best fit to find RC time','fontsize',fsize)
fig_V_over_I = figure; ax_V_over_I = axes('nextplot','add'); grid on; set(gca,'fontsize',fsize);
xlabel('t/s','fontsize',fsize); ylabel('R = V/I /\Omega','fontsize',fsize); 
title('R = V/I for capacitor discharge','fontsize',fsize)

%Determine legend string
lgnd_str = {};
for n=1:length(R)
    lgnd_str{n} = ['R=',num2str(R(n)),'\Omega'];
end

%Determine plot colours from current colormap
new_map = interp_colormap(length(R));

%For each run (i.e. discharge resistance)
% (i)   Determine time when charging started
% (ii)  Determine time when discharge started
% (iii) Separate charge and discharge, with time starting from zero for
%       both. This will enable traces to be compared.
% (iv)  Perform a line of best fit for each discharge (log V vs t) and
%       and determine RC time. R = R_internal + R_added.
% (v)   Plot RC vs R_added, which will yield C (the gradient) and also
%       R_internal.
for n=1:length(data)
    
    %Split charge and discharge curves
    [data(n).tc,data(n).Vc,data(n).td,data(n).Vd,data(n).Id] = ...
        split_charge_and_discharge( data(n).t_s,data(n).V_volts,data(n).I_mA );
    
    %Perform line of best fit on discharge curve
    y = log(data(n).Vd); y(imag(y)~=0) = NaN; 
    [yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc( data(n).td, y);
    RC(n) = -1/m;
    V0(n) = exp(c);
    
    %Determine graph colour
    r = new_map(n,:);
    
    %Add to charging voltage graph
    figure(fig_Vcharge); axes(ax_V_charge);
    plot( data(n).tc, data(n).Vc, 'color',r );
    
    %Add to charging voltage graph
    figure(fig_Vdischarge); axes(ax_V_discharge);
    plot( data(n).td, data(n).Vd, 'color',r );
    
    %Add to discharging current graph
    figure(fig_Idischarge); axes(ax_I_discharge);
    plot( data(n).td, data(n).Id, 'color',r );
    
    %Add to bestfit graph
    figure(fig_best_fit); axes(ax_best_fit);
    plot( xfit,yfit,'-', 'color',r );
    plot( data(n).td,y,'+', 'color',r ); 
    
    %Add to R = V/I graph
    figure(fig_V_over_I); axes(ax_V_over_I);
    plot( data(n).td, data(n).Vd./(1e-3*data(n).Id),'-', 'color',r); 
end

%Save graphs as PNG files
figure(fig_Vcharge); xlim([0,50]); ylim([0,8]); legend( lgnd_str,'fontsize',fsize*0.7 );
box on; print( gcf, 'charging voltage vs time.png','-dpng','-r300' ); close(gcf); 
figure(fig_Vdischarge); xlim([0,100]); ylim([0,7]); legend( lgnd_str,'fontsize',fsize*0.7 );
box on; print( gcf, 'discharging voltage vs time.png','-dpng','-r300' ); close(gcf);
figure(fig_Idischarge); xlim([0,100]); ylim([0,200]); legend( lgnd_str,'fontsize',fsize*0.7 );
box on; print( gcf, 'discharging current vs time.png','-dpng','-r300' ); close(gcf);
figure(fig_best_fit); xlim([0,100]);
box on;  print( gcf, 'ln(V) vs t best fit.png','-dpng','-r300' ); close(gcf);
figure(fig_V_over_I); xlim([0,100]); ylim([0,200]); legend( lgnd_str,'fontsize',fsize*0.7 );
box on;  print( gcf, 'V over I vs t.png','-dpng','-r300' ); close(gcf);

%Plot RC vs Radded
[yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc( R,RC );
figure; plot( R, RC, '+' ); hold on; plot( xfit,yfit ); grid on;
set(gca,'fontsize',fsize);
xlabel('Added resistance /\Omega','fontsize',fsize);
ylabel('RC from capacitor decay /s','fontsize',fsize);
title(['C= (',num2str( m,4 ),' +/- ',num2str(dm,2), ')F, R_{int}= (',...
    num2str(c/m,3),' +/- ',num2str(dc/m,3),')\Omega.'],'fontsize',fsize );
print( gcf,'RC vs R.png','-dpng','-r300' ); close(gcf);

%%

%Split charge and discharge
function [tc,Vc,td,Vd,Id] = split_charge_and_discharge( t,V,I )

%Find charge and discharge times
tcharge = -1; tchargemax = -1; tdischarge = -1;
for n=3:length(t)-2
    
    %Step through data and when there are three consecutive increases, the
    %first point is the start of the charge.
    if ( V(n)>V(n-1) ) && ( V(n-1)>V(n-2) ) && ( V(n-2)>V(n-3) ) && ( tcharge ==-1 )
        tcharge = t(n-3);
        ncharge = n-3;
    end
    
    %When three consecutive decreases, the first point is the start of the
    %discharge.
    if ( V(n)<V(n-1) ) && ( V(n-1)<V(n-2) ) && ( V(n-2)<V(n-3) ) && ( tchargemax ==-1 )
        tchargemax = t(n-3);
        nchargemax= n-3;
    end
    
    %When three consecutive decreases, the first point is the start of the
    %discharge.
    if ( V(n)<V(n-1) ) && ( V(n-1)<V(n-2) ) && ( V(n-2)<V(n-3) ) && ( tdischarge ==-1 ) && ( V(n-3)<7.0 )
        tdischarge = t(n-3);
        ndischarge = n-3;
    end
end

%Split charge and discharge
tc = t(ncharge:nchargemax) - tcharge;
Vc = V(ncharge:nchargemax);
td = t(ndischarge:end) - tdischarge;
Vd = V(ndischarge:end);
Id = I(ndischarge:end);

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];
n = length(x);

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = COVxy/Vx; c = ybar - m*xbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x + c; xfit = x;

%Compute errors in gradient m and intercept c
s = sqrt( (1/(n-2))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);
dc = ( s/sqrt(n) )*sqrt( 1 + (xbar^2)/Vx );

%Determine envelope lines
yupper = (m+dm)*x + c - dc;
ylower = (m-dm)*x + c + dc;

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008

function new_map = interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code